@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-__version__ = '0.2'
+__version__ = '0.3'
 
 import re
 
@@ -27,33 +27,54 @@ class Address(object):
     NAME  = 2
     UNIT  = 3
 
-    PAIRS_TO_NORMALIZE = (
-        # the chars to remove
-        (u' ' , u''),
-        (u'　', u''),
-        (u',' , u''),
-        (u'，', u''),
-        # unity the chars
-        (u'台', u'臺'),
-        (u'-' , u'之'),
-        (u'０', u'0'),
-        (u'１', u'1'),
-        (u'２', u'2'),
-        (u'３', u'3'),
-        (u'４', u'4'),
-        (u'５', u'5'),
-        (u'６', u'6'),
-        (u'７', u'7'),
-        (u'８', u'8'),
-        (u'９', u'9'),
-    )
+    TO_REPLACE_RE = re.compile(u'''
+    [ 　,，台-]
+    |
+    [０-９]
+    |
+    [一二三四五六七八九]?
+    十?
+    [一二三四五六七八九]
+    (?=[段路街巷弄號樓])
+    ''', re.X)
+
+    TO_REMOVE_SET = set(u' 　,，')
+
+    TO_REPLACE_MAP = {
+        u'-' : u'之', u'台': u'臺',
+        u'一': u'1', u'二': u'2', u'三': u'3', u'四': u'4', u'五': u'5',
+        u'六': u'6', u'七': u'7', u'八': u'8', u'九': u'9',
+    }
 
     @staticmethod
     def normalize(s):
+
         if isinstance(s, str):
             s = s.decode('utf-8')
-        for from_, to in Address.PAIRS_TO_NORMALIZE:
-            s = s.replace(from_, to)
+
+        def replace(m):
+
+            found = m.group()
+
+            if found in Address.TO_REMOVE_SET:
+                return u''
+            if found in Address.TO_REPLACE_MAP:
+                return Address.TO_REPLACE_MAP[found]
+
+            len_found = len(found)
+
+            # 65296 = '０'; 65305 = '９'; 65248 = '０'-'0'
+            if len_found == 1 and 65296 <= ord(found) <= 65305:
+                return unichr(ord(found)-65248)
+
+            # for '十一' to '九十九'
+            if len_found == 2:
+                return u'1'+Address.TO_REPLACE_MAP[found[1]]
+            if len_found == 3:
+                return Address.TO_REPLACE_MAP[found[0]]+Address.TO_REPLACE_MAP[found[2]]
+
+        s = Address.TO_REPLACE_RE.sub(replace, s)
+
         return s
 
     @staticmethod
@@ -75,7 +96,7 @@ class Address(object):
     def __repr__(self):
         return 'Address(%r)' % self.flat()
 
-    def parse_no_token(self, idx):
+    def parse_token(self, idx):
         try:
             token = self.tokens[idx]
         except IndexError:
@@ -137,23 +158,26 @@ class Rule(Address):
     def match(self, addr):
 
         # the part reserves for rule tokens
-        my_last_idx = len(self.tokens)
-        my_last_idx -= bool(self.rule_tokens) and u'全' not in self.rule_tokens
-        my_last_idx -= u'至' in self.rule_tokens
+        my_end_pos = len(self.tokens)
+        my_end_pos -= bool(self.rule_tokens) and u'全' not in self.rule_tokens
+        my_end_pos -= u'至' in self.rule_tokens
 
-        his_last_idx = len(addr.tokens)
-        his_last_idx -= bool(self.rule_tokens)
+        his_end_pos = len(addr.tokens)
+        his_end_pos -= bool(self.rule_tokens)
 
-        my_tokens_to_match = self.tokens[:my_last_idx]
+        my_tokens_to_match = self.tokens[:my_end_pos]
         if my_tokens_to_match:
 
             # the addr's tokens whose unit bigger than rule's are ignorable
             start_unit = my_tokens_to_match[0][Address.UNIT]
-            for his_start_idx, his_token in enumerate(addr.tokens):
-                if his_token[Address.UNIT] == start_unit:
+            his_start_pos = 0
+            for his_token in addr.tokens:
+                if his_token[Address.UNIT] != start_unit:
+                    his_start_pos += 1
+                else:
                     break
 
-            his_tokens_to_match = addr.tokens[his_start_idx:his_last_idx]
+            his_tokens_to_match = addr.tokens[his_start_pos:his_end_pos]
             if len(my_tokens_to_match) != len(his_tokens_to_match):
                 return False
 
@@ -162,9 +186,9 @@ class Rule(Address):
                     return False
 
         # check the rule tokens
-        his_no_pair     = addr.parse_no_token(-1)
-        my_no_pair      = self.parse_no_token(-1)
-        my_asst_no_pair = self.parse_no_token(-2)
+        his_no_pair     = addr.parse_token(-1)
+        my_no_pair      = self.parse_token(-1)
+        my_asst_no_pair = self.parse_token(-2)
         for rt in self.rule_tokens:
             if (
                 (rt == u'全'         and not his_no_pair > (0, 0)) or
@@ -194,9 +218,10 @@ class Directory(object):
     @staticmethod
     def get_common_part(str_a, str_b):
 
-        if not str_a: return str_b
-        if not str_b: return str_a
+        if str_a is None: return str_b
+        if str_b is None: return str_a
 
+        i = 0 # for the case range is empty
         for i in range(min(len(str_a), len(str_b))):
             if str_a[i] != str_b[i]:
                 break
@@ -211,7 +236,7 @@ class Directory(object):
         self.tokens_rzpairs_map = defaultdict(list)
 
         # gzipcode = gradual_zipcode
-        self.tokens_gzipcode_map = defaultdict(str)
+        self.tokens_gzipcode_map = defaultdict(lambda: None)
 
     def load(self, zipcode, addr_str, tail_rule_str):
 
@@ -220,11 +245,15 @@ class Directory(object):
         # (a, b, c)
         self.tokens_rzpairs_map[tokens].append((addr_str+tail_rule_str, zipcode))
 
-        # (a, b, c) -> (a, b, c) ... (a,)
-        for i in range(len(tokens), 0, -1):
-            k = tokens[:i]
-            orig = self.tokens_gzipcode_map[k]
-            self.tokens_gzipcode_map[k] = Directory.get_common_part(orig, zipcode)
+        # (a, b, c) -> (a,); (a, b); (a, b, c); (b,); (b, c); (c,)
+        len_tokens = len(tokens)
+        for f in range(len_tokens):
+            for l in range(f, len_tokens):
+                for s in range(1, 2+(f == 0 and l == 2)):
+                    k = tokens[f:l+1:s]
+                    orig = self.tokens_gzipcode_map[k]
+                    comm = Directory.get_common_part(orig, zipcode)
+                    self.tokens_gzipcode_map[k] = comm
 
     def load_chp_csv(self, lines, skip_first=True):
 
